import { Modal } from 'ant-design-vue'
import api from './api'
import request, { retry } from './request'

export let $config = undefined
export let $store = undefined
export let $router = undefined

export const isObject = (val) => {
  return val !== null && typeof val === 'object'
}

export const isString = (val) => typeof val === 'string'

/**
 * 把横杠命名方式转换成驼峰命名方式
 */
export const camelize = (str) => {
  return str.replace(/-(\w)/g, (_, c) => (c ? c.toUpperCase() : ''))
}

/**
 * 首字母大写
 */
export const capitalize = (str) => str.charAt(0).toUpperCase() + str.slice(1)

/**
 * 添加 on 前缀，并且首字母大写
 */
export const toHandlerKey = (str) => (str ? `on${capitalize(str)}` : ``)

/**
 * 下载文件
 */
export const downloadFile = (url, filename) => {
  return new Promise((resolve) => {
    function xhrDownload(objectURL) {
      const link = document.createElement('a')
      const body = document.querySelector('body')
      link.href = objectURL
      link.download = filename
      link.style.display = 'none'
      body.appendChild(link)
      link.click()
      body.removeChild(link)
      window.URL.revokeObjectURL(objectURL)
      resolve()
    }
    if (url instanceof File || url instanceof Blob) {
      xhrDownload(window.URL.createObjectURL(url))
    } else if (typeof url === 'string' && url.startsWith('blob:')) {
      xhrDownload(url)
    } else {
      const xhr = new XMLHttpRequest()
      xhr.open('GET', `${url}?t=${String(Date.now()).slice(0, 8)}`, true)
      xhr.responseType = 'blob'
      xhr.onload = function () {
        if (xhr.status === 200) {
          if (!filename) {
            const arr = url.split('/')
            filename = arr.length && arr.pop()
          }
          xhrDownload(window.URL.createObjectURL(xhr.response))
        }
      }
      xhr.send()
    }
  })
}

export const validate = {
  idCard(num = '') {
    num = num.toUpperCase()
    if (!/(^\d{15}$)|(^\d{17}([0-9]|X)$)/.test(num)) {
      return false
    }
    var len, re
    len = num.length
    if (len == 15) {
      re = new RegExp(/^(\d{6})(\d{2})(\d{2})(\d{2})(\d{3})$/)
      var arrSplit = num.match(re)
      var dtmBirth = new Date('19' + arrSplit[2] + '/' + arrSplit[3] + '/' + arrSplit[4])
      var bCorrectDay
      bCorrectDay =
        dtmBirth.getYear() == Number(arrSplit[2]) &&
        dtmBirth.getMonth() + 1 == Number(arrSplit[3]) &&
        dtmBirth.getDate() == Number(arrSplit[4])
      if (!bCorrectDay) {
        return false
      } else {
        var arrInt = new Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2)
        var arrCh = new Array('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2')
        var nTemp = 0,
          i
        num = num.substr(0, 6) + '19' + num.substr(6, num.length - 6)
        for (i = 0; i < 17; i++) {
          nTemp += num.substr(i, 1) * arrInt[i]
        }
        num += arrCh[nTemp % 11]
        return true
      }
    }
    if (len == 18) {
      re = new RegExp(/^(\d{6})(\d{4})(\d{2})(\d{2})(\d{3})([0-9]|X)$/)
      arrSplit = num.match(re)
      dtmBirth = new Date(arrSplit[2] + '/' + arrSplit[3] + '/' + arrSplit[4])
      bCorrectDay
      bCorrectDay =
        dtmBirth.getFullYear() == Number(arrSplit[2]) &&
        dtmBirth.getMonth() + 1 == Number(arrSplit[3]) &&
        dtmBirth.getDate() == Number(arrSplit[4])
      if (!bCorrectDay) {
        return false
      } else {
        var valnum
        arrInt = new Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2)
        arrCh = new Array('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2')
        ;(nTemp = 0), i
        for (i = 0; i < 17; i++) {
          nTemp += num.substr(i, 1) * arrInt[i]
        }
        valnum = arrCh[nTemp % 11]
        if (valnum != num.substr(17, 1)) {
          return false
        }
        return true
      }
    }
    return false
  }, // 国标身份证
  email(str) {
    const reg = /^\w+@[a-zA-Z0-9]{2,10}(?:\.[a-z]{2,4}){1,3}$/
    return reg.test(str)
  }, //邮箱
  phone(str) {
    const reg = /^1\d{10}$/
    return reg.test(str)
  }, // 手机号
  carNumber(str) {
    const reg =
      /^[京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领][A-HJ-NP-Z](([A-HJ-NP-Z][A-HJ-NP-Z0-9]{4})|([A-HJ-NP-Z0-9]{4}))[A-HJ-NP-Z0-9挂学警港澳使领]$/
    return reg.test(str)
  }, //车牌号
  USCI(str) {
    const reg = /^[^_IOZSVa-z\W]{2}\d{6}[^_IOZSVa-z\W]{10}$/g
    return reg.test(str)
  }, // 统一社会信用代码
  GATCard(str) {
    const reg = /^([A-Z]\d{6,10}(\(\w{1}\))?)$/
    const reg2 = /^\d{8}|^[a-zA-Z0-9]{10}|^\d{18}$/
    return reg.test(str) || reg2.test(str)
  }, // 港澳台通行证
  passport(str) {
    var reg = /^([a-zA-z]|[0-9]){5,17}$/
    return reg.test(str)
  }, // 护照
}

export const localRun = async (app, config) => {
  $config = config
  $store = config && config.store
  $router = config && config.router
  //本地开发
  const localhosts = ['localhost', '127', '172.16.191']
  if (config) {
    //如果传了配置
    if (localhosts.some((host) => location.host.startsWith(host))) {
      setTimeout(() => {
        const appElement = document.querySelector('#app')
        if (appElement) {
          appElement.style.backgroundColor = '#f0f2f5'
        }
      })
      request.defaults.baseURL = config.baseURL
      if (config.token) {
        request.defaults.headers.common['Authorization'] = config.token
      } else if (config.username && config.password) {
        //TODO 如果传了账号密码
      }
      const APP_CONFIG = {
        dictionary: null,
        user: null,
        allAssetCommunityTreeList: null,
        token: config.token,
        baseURL: config.baseURL,
      }
      APP_CONFIG.user = await api.getCurrentUser({ tenantId: 'admin', client: 'community' })
      APP_CONFIG.allAssetCommunityTreeList = window.allAssetCommunityTreeList =
        await api.getAllAssetCommunityTreeList({})
      await api.getSysDict().then((res) => {
        const dictObj = {}
        res.forEach((item) => {
          item.sysDictInfoList.forEach((ele) => {
            ele['sysDictValueModels'].forEach((e) => {
              e.value = e.value + ''
              e.name = e.key
            })
            dictObj[ele.code] = ele['sysDictValueModels']
          })
          APP_CONFIG.dictionary = dictObj
        })
      })
      if ($store) {
        $store.commit('SET_APP_CONFIG', APP_CONFIG)
      }
    } else {
      // 非本地开发
      window.addEventListener('message', (event) => {
        const { data } = event
        if (data.type === 'getAppConfig' && data.APP_CONFIG) {
          window.allAssetCommunityTreeList = data.APP_CONFIG.allAssetCommunityTreeList
          if (config && $store) {
            $store.commit('SET_APP_CONFIG', data.APP_CONFIG)
          }
          request.defaults.baseURL = data.APP_CONFIG.baseURL
          request.defaults.headers.common['Authorization'] = data.APP_CONFIG.token
        }
        if (data.type === 'getNewToken') {
          request.defaults.headers.common['Authorization'] = data.APP_CONFIG.token
          retry()
        }
      })
      window.parent.postMessage({ type: 'getAppConfig' }, '*')
    }
  }
  if (config && $router) {
    //加埋点
    $router.beforeEach((to, from, next) => {
      Modal.destroyAll()
      next()
    })
    $router.afterEach(() => {
      setTimeout(() => {
        messageForBuryingPoints({
          eventCode: 'PAGE_VIEW',
          options: {
            newPage: getCurrentPageInfo().path,
          },
        })
      })
    })
  }
}

//配置antd
export const setAntdConfig = (app) => {
  const notification = app.prototype.$notification
  notification.config({
    closeIcon: (h) =>
      h('span', {
        class: 'iconfont icon-guanbi_line',
        style: {
          fontSize: '16px',
        },
      }),
  })
  const originSuccess = notification.success
  const originInfo = notification.info
  const originWarning = notification.warning
  const originError = notification.error
  notification.success = (...args) => {
    originSuccess({
      icon: (h) =>
        h('span', {
          class: 'iconfont icon-yuan_duihao_fill',
          style: {
            fontSize: '22px',
            color: '#23c172',
          },
        }),
      ...args[0],
    })
  }
  notification.info = (...args) => {
    originInfo({
      icon: (h) =>
        h('span', {
          class: 'iconfont icon-yuan_tanhao_fill',
          style: {
            fontSize: '22px',
            color: '#2c8df4',
          },
        }),
      ...args[0],
    })
  }
  notification.warning = (...args) => {
    originWarning({
      icon: (h) =>
        h('span', {
          class: 'iconfont icon-yuan_tanhao_fill',
          style: {
            fontSize: '22px',
            color: '#faad14',
          },
        }),
      ...args[0],
    })
  }
  notification.error = (...args) => {
    originError({
      icon: (h) =>
        h('span', {
          class: 'iconfont icon-yuan_tanhao_fill',
          style: {
            fontSize: '22px',
            color: '#fa5f5f',
          },
        }),
      ...args[0],
    })
  }
}

/**
 * 发送埋点信息到父级
 * @param eventCode
 * @param needParams
 * @param options
 */
export const messageForBuryingPoints = ({ eventCode, needParams = {}, options = {} }) => {
  console.log('messageForBuryingPoints', {
    type: 'buryingPoint',
    eventCode,
    needParams,
    options,
  })
  window.parent.postMessage(
    {
      type: 'buryingPoint',
      eventCode,
      needParams,
      options,
    },
    '*'
  )
}

/**
 * 获取当前页面
 */
export const getCurrentPageInfo = () => {
  return {
    path: window.location.href,
  }
}

/**
 * 点击应用埋点
 */
export const buttonClickBuryingPoint = () => {
  messageForBuryingPoints({
    eventCode: 'BUTTON_CLICK',
    needParams: {
      pageCode: getCurrentPageInfo().path,
    },
  })
}

/**
 * 搜索应用埋点
 */
export const appSearchBuryingPoint = (keywords) => {
  const params = {}
  if (keywords instanceof Array) {
    params.options = { keywords }
  } else {
    params.needParams = { keywords }
  }
  messageForBuryingPoints({
    eventCode: 'APP_SEARCH',
    ...params,
  })
}

/**
 * 报错埋点
 */
export const requestErrorPoint = (error) => {
  messageForBuryingPoints({
    eventCode: 'APP_ERROR',
    needParams: {
      errorInfo: JSON.stringify((error.response && error.response.data) || {}),
      url: error.config && error.config.url,
    },
    options: {
      errorCode: error.response && error.response.data.code,
    },
  })
}
