import axios from 'axios'
import { notification } from 'ant-design-vue'
// import CryptoJS from 'crypto-js'
import { messageForBuryingPoints } from './index.js'
// // 需要加密的请求类型
// const needSignMethods = ['post', 'put', 'patch']
// // 加密key
// const aesKey = CryptoJS.enc.Utf8.parse('NL7Rr1J3H/IrgAM++2LVkw==')
import { $config } from './index.js'

const instance = axios.create({
  baseURL: '',
  headers: {},
})

let retryRequests = [] // 储存未执行的请求
let isRefreshing = false // 记录刷新token状态

// 刷新 token 以后执行未完成的请求
export const retry = () => {
  retryRequests.forEach((callback) => {
    callback(instance.defaults.headers.common['Authorization'])
  })
  retryRequests = []
  isRefreshing = false
}

const { CancelToken } = axios
// 请求队列，缓存发出的请求
const cacheRequest = {
  ifCancel: false,
  cancelRequest: [],
}

instance.interceptors.request.use(
  (config) => {
    // if (
    //   config.method &&
    //   needSignMethods.indexOf(config.method) !== -1 &&
    //   config.isSign !== true &&
    //   config.data &&
    //   window.APP_CONFIG &&
    //   window.APP_CONFIG.needSign
    // ) {
    //   const originData = config.data
    //   const message = CryptoJS.enc.Utf8.parse(JSON.stringify(originData))
    //   const cfg = {
    //     format: CryptoJS.format.Hex,
    //     mode: CryptoJS.mode.ECB,
    //     padding: CryptoJS.pad.Pkcs7,
    //   }
    //   const encrypted = CryptoJS.AES.encrypt(message, aesKey, cfg)
    //   const ciphertext = encrypted.ciphertext.toString()
    //   config.data = {
    //     gtSign: ciphertext,
    //     data: originData,
    //   }
    //   config.isSign = true
    // }
    config.cancelToken = new CancelToken((c) => {
      cacheRequest.cancelRequest.push(c)
    })
    return config
  },
  (error) => {
    notification.error({
      message: '提示!',
      description: error.message,
    })
    return Promise.reject(error.message)
  }
)

instance.interceptors.response.use(
  (response) => {
    //response.data为字符串或Blob对象的情况直接返回
    if (typeof response.data === 'string' || response.data instanceof Blob) {
      return response.data
    }
    //response.data为对象的情况
    const { code, message } = response.data
    if (code === 0) {
      // 请求成功
      cacheRequest.ifCancel = false
      cacheRequest.cancelRequest = []
      return response.data.data
    }
    if ($config) {
      // 状态码 200，但是 token 过期
      if (code === 2004) {
        if (!isRefreshing) {
          isRefreshing = true
          window.parent.postMessage({ code: '2004' }, '*') // token 过期，向父级窗口发送消息告知刷新 token
        }
        return new Promise((resolve) => {
          //  将当前未执行成功的请求重新发起，push 到 retryRequests
          retryRequests.push((token) => {
            resolve(
              instance({
                ...response.config,
                headers: { ...response.config.headers, Authorization: token },
              })
            )
          })
        })
      }
      // 如果被踢出，取消队列请求
      if (cacheRequest.ifCancel) {
        return cacheRequest.cancelRequest.forEach((canceler) => canceler())
      }
      // 踢出、权限变更，触发底座退出登录方法
      if ([2005, 2006, 2007].includes(code)) {
        cacheRequest.ifCancel = true
        return window.parent.postMessage({ code, message }, '*')
      }
    }
    if (code !== 0) {
      notification.error({
        message: `提示:${code}`,
        description: message || '请求失败！',
      })
      return Promise.reject(response.data.message)
    }
  },
  (error) => {
    console.log(error)
    const { code, message } = error.response.data
    // 状态码非 200
    notification.error({
      message: `提示:${code || error.response.status}`,
      description: message || error.message,
    })
    requestErrorPoint(error)
    return Promise.reject(error.message)
  }
)

//报错埋点
function requestErrorPoint(error) {
  messageForBuryingPoints({
    eventCode: 'APP_ERROR',
    needParams: {
      errorInfo: JSON.stringify(error.response.data || {}),
      url: error.config.url,
    },
    options: {
      errorCode: error.response.data.code,
    },
  })
}

export default instance
